@@ -19,17 +19,17 @@
 #ifndef _HEDGEHOGER_WIDGET
 #define _HEDGEHOGER_WIDGET
 
-#include <QWidget>
+#include "itemNum.h"
 
 class FrameTeams;
 
-class CHedgehogerWidget : public QWidget
+class CHedgehogerWidget : public ItemNum
 {
   Q_OBJECT
 
  public:
-  CHedgehogerWidget(QWidget * parent);
-  ~CHedgehogerWidget();
+  CHedgehogerWidget(const QImage& im, QWidget * parent);
+  virtual ~CHedgehogerWidget();
   unsigned char getHedgehogsNum() const;
   void setHHNum (unsigned int num);
   void setNonInteractive();
@@ -38,14 +38,12 @@ class CHedgehogerWidget : public QWidget
   void hedgehogsNumChanged();
 
  protected:
-  virtual void paintEvent(QPaintEvent* event);
-  virtual void mousePressEvent ( QMouseEvent * event );
+  virtual void incItems();
+  virtual void decItems();
 
  private:
   CHedgehogerWidget();
-  unsigned char numHedgehogs;
   FrameTeams* pOurFrameTeams;
-  bool nonInteractive;
 };
 
 #endif // _HEDGEHOGER_WIDGET
