@@ -16,57 +16,65 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include <QtGui>
-#include <QStringList>
-#include <QProcess>
-#include <QDir>
-#include <QPixmap>
-#include <QRegExp>
-#include <QIcon>
 #include <QFile>
 #include <QTextStream>
 #include <QMessageBox>
+#include <QPushButton>
+#include <QListWidget>
+#include <QStackedLayout>
+#include <QLineEdit>
+#include <QLabel>
+#include <QRadioButton>
+#include <QSpinBox>
+#include <QCloseEvent>
 
 #include "hwform.h"
 #include "game.h"
 #include "team.h"
 #include "teamselect.h"
+#include "selectWeapon.h"
 #include "gameuiconfig.h"
 #include "pages.h"
 #include "hwconsts.h"
 #include "newnetclient.h"
 #include "gamecfgwidget.h"
+#include "netserverslist.h"
 #include "netudpserver.h"
-#include "netudpwidget.h"
+#include "netwwwserver.h"
 #include "chatwidget.h"
+#include "playrecordpage.h"
+#include "input_ip.h"
 
 HWForm::HWForm(QWidget *parent)
-  : QMainWindow(parent), pnetserver(0), pUdpServer(0)
+  : QMainWindow(parent), pnetserver(0), pRegisterServer(0), editedTeam(0), hwnet(0)
 {
 	ui.setupUi(this);
-
 	config = new GameUIConfig(this, cfgdir->absolutePath() + "/hedgewars.ini");
 
 	UpdateTeamsLists();
+	UpdateWeapons();
 
 	connect(ui.pageMain->BtnSinglePlayer,	SIGNAL(clicked()),	this, SLOT(GoToSinglePlayer()));
 	connect(ui.pageMain->BtnSetup,	SIGNAL(clicked()),	this, SLOT(GoToSetup()));
 	connect(ui.pageMain->BtnMultiplayer,	SIGNAL(clicked()),	this, SLOT(GoToMultiplayer()));
+	connect(ui.pageMain->BtnLoad,	SIGNAL(clicked()),	this, SLOT(GoToSaves()));
 	connect(ui.pageMain->BtnDemos,	SIGNAL(clicked()),	this, SLOT(GoToDemos()));
 	connect(ui.pageMain->BtnNet,	SIGNAL(clicked()),	this, SLOT(GoToNet()));
 	connect(ui.pageMain->BtnInfo,	SIGNAL(clicked()),	this, SLOT(GoToInfo()));
-	connect(ui.pageMain->BtnExit, SIGNAL(clicked()), this, SLOT(close()));
+	connect(ui.pageMain->BtnExit, SIGNAL(pressed()), this, SLOT(btnExitPressed()));
+	connect(ui.pageMain->BtnExit, SIGNAL(clicked()), this, SLOT(btnExitClicked()));
 
-	connect(ui.pageLocalGame->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageLocalGame->BtnSimpleGame,	SIGNAL(clicked()),	this, SLOT(SimpleGame()));
+	connect(ui.pageSimpleGame->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageSimpleGame->BtnSimpleGame,	SIGNAL(clicked()),	this, SLOT(SimpleGame()));
 
 	connect(ui.pageEditTeam->BtnTeamSave,	SIGNAL(clicked()),	this, SLOT(TeamSave()));
 	connect(ui.pageEditTeam->BtnTeamDiscard,	SIGNAL(clicked()),	this, SLOT(TeamDiscard()));
 
 	connect(ui.pageMultiplayer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 	connect(ui.pageMultiplayer->BtnStartMPGame,	SIGNAL(clicked()),	this, SLOT(StartMPGame()));
-	connect(ui.pageMultiplayer->teamsSelect, SIGNAL(setEnabledGameStart(bool)), 
+	connect(ui.pageMultiplayer->teamsSelect, SIGNAL(setEnabledGameStart(bool)),
 		ui.pageMultiplayer->BtnStartMPGame, SLOT(setEnabled(bool)));
+	connect(ui.pageMultiplayer->teamsSelect, SIGNAL(SetupClicked()), this, SLOT(IntermediateSetup()));
 
 	connect(ui.pagePlayDemo->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 	connect(ui.pagePlayDemo->BtnPlayDemo,	SIGNAL(clicked()),	this, SLOT(PlayDemo()));
@@ -78,30 +86,83 @@ HWForm::HWForm(QWidget *parent)
 	connect(ui.pageOptions->BtnSaveOptions,	SIGNAL(clicked()),	config, SLOT(SaveOptions()));
 	connect(ui.pageOptions->BtnSaveOptions,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 
+	connect(ui.pageOptions->WeaponEdit,	SIGNAL(clicked()),	this, SLOT(GoToSelectWeapon()));
+	connect(ui.pageOptions->WeaponsButt,	SIGNAL(clicked()),	this, SLOT(GoToSelectNewWeapon()));
+	connect(ui.pageSelectWeapon->pWeapons,       SIGNAL(weaponsChanged()), this, SLOT(UpdateWeapons()));
+	connect(ui.pageNetGame->pGameCFG,       SIGNAL(newWeaponsName(const QString&)), this, SLOT(NetWeaponNameChanged(const QString&)));
+
 	connect(ui.pageNet->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageNet->BtnNetConnect,	SIGNAL(clicked()),	this, SLOT(NetConnect()));
-	connect(ui.pageNet->BtnNetSvrStart, SIGNAL(clicked()), this, SLOT(NetStartServer()));
-	connect(ui.pageMain->BtnNet,	SIGNAL(clicked()), ui.pageNet->pUdpClient, SLOT(updateList()));
-	connect(ui.pageNet->pUpdateUdpButt, SIGNAL(clicked()), ui.pageNet->pUdpClient, SLOT(updateList()));
-	connect(ui.pageNet->pUdpClient->serversList,	SIGNAL(doubleClicked (const QModelIndex &)),	this, SLOT(NetConnectServer()));
+	connect(ui.pageNet->BtnSpecifyServer,	SIGNAL(clicked()),	this, SLOT(NetConnect()));
+	connect(ui.pageNet->BtnNetSvrStart,	SIGNAL(clicked()),	this, SLOT(GoToNetServer()));
+	connect(ui.pageNet,	SIGNAL(connectClicked(const QString &, quint16)),	this, SLOT(NetConnectServer(const QString &, quint16)));
+	
+	connect(ui.pageNetServer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageNetServer->BtnStart,	SIGNAL(clicked()),	this, SLOT(NetStartServer()));
 
 	connect(ui.pageNetGame->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 	connect(ui.pageNetGame->BtnGo,	SIGNAL(clicked()),	this, SLOT(NetStartGame()));
-	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(setEnabledGameStart(bool)), 
+	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(setEnabledGameStart(bool)),
 		ui.pageNetGame->BtnGo, SLOT(setEnabled(bool)));
+	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(SetupClicked()), this, SLOT(IntermediateSetup()));
 
 	connect(ui.pageInfo->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 
 	connect(ui.pageGameStats->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 
-	connect(ui.pageMultiplayer->teamsSelect, SIGNAL(NewTeam()), this, SLOT(NewTeam()));
+	connect(ui.pageSinglePlayer->BtnSimpleGamePage,	SIGNAL(clicked()),	this, SLOT(GoToSimpleGame()));
+	connect(ui.pageSinglePlayer->BtnTrainPage,	SIGNAL(clicked()),	this, SLOT(GoToTraining()));
+	connect(ui.pageSinglePlayer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+
+	connect(ui.pageTraining->BtnStartTrain,	SIGNAL(clicked()),	this, SLOT(StartTraining()));
+	connect(ui.pageTraining->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+
+	connect(ui.pageSelectWeapon->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageSelectWeapon->BtnDefault,	SIGNAL(clicked()),	ui.pageSelectWeapon->pWeapons, SLOT(setDefault()));
+	connect(ui.pageSelectWeapon->BtnSave,	SIGNAL(clicked()),	ui.pageSelectWeapon->pWeapons, SLOT(save()));
+
+	connect(ui.pageSelectWeapon->BtnDelete,	SIGNAL(clicked()),	
+		ui.pageSelectWeapon->pWeapons, SLOT(deleteWeaponsName())); // executed first
+	connect(ui.pageSelectWeapon->pWeapons,	SIGNAL(weaponsDeleted()),
+		this, SLOT(UpdateWeapons())); // executed second
+	connect(ui.pageSelectWeapon->pWeapons,	SIGNAL(weaponsDeleted()),
+		this, SLOT(GoBack())); // executed third
 
 	GoToPage(ID_PAGE_MAIN);
 }
 
-void HWForm::UpdateTeamsLists()
+void HWForm::UpdateWeapons()
 {
-	QStringList teamslist = config->GetTeamsList();
+  // FIXME: rewrite this with boost (or TR1/0x)
+  QVector<QComboBox*> combos;
+  combos.push_back(ui.pageOptions->WeaponsName);
+  combos.push_back(ui.pageSimpleGame->gameCFG->WeaponsName);
+  combos.push_back(ui.pageMultiplayer->gameCFG->WeaponsName);
+  combos.push_back(ui.pageNetGame->pGameCFG->WeaponsName);
+
+  for(QVector<QComboBox*>::iterator it=combos.begin(); it!=combos.end(); ++it) {
+    (*it)->clear();
+    (*it)->addItems(ui.pageSelectWeapon->pWeapons->getWeaponNames());
+    int pos=(*it)->findText("Default");
+    if (pos!=-1) {
+      (*it)->setCurrentIndex(pos);
+    }
+  }
+}
+
+void HWForm::NetWeaponNameChanged(const QString& name)
+{
+  QString ammo=ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageNetGame->pGameCFG->WeaponsName->currentText());
+  hwnet->onWeaponsNameChanged(name, ammo);
+}
+
+void HWForm::UpdateTeamsLists(const QStringList* editable_teams)
+{
+	QStringList teamslist;
+	if(editable_teams) {
+	  teamslist=*editable_teams;
+	} else {
+	  teamslist = config->GetTeamsList();
+	}
 
 	if(teamslist.empty()) {
 		HWTeam defaultTeam("DefaultTeam");
@@ -123,11 +184,33 @@ void HWForm::GoToSinglePlayer()
 	GoToPage(ID_PAGE_SINGLEPLAYER);
 }
 
+void HWForm::GoToSimpleGame()
+{
+	GoToPage(ID_PAGE_SIMPLEGAME);
+}
+
+void HWForm::GoToTraining()
+{
+	GoToPage(ID_PAGE_TRAINING);
+}
+
 void HWForm::GoToSetup()
 {
 	GoToPage(ID_PAGE_SETUP);
 }
 
+void HWForm::GoToSelectNewWeapon()
+{
+  ui.pageSelectWeapon->pWeapons->setWeaponsName("", false);
+	GoToPage(ID_PAGE_SELECTWEAPON);
+}
+
+void HWForm::GoToSelectWeapon()
+{
+  ui.pageSelectWeapon->pWeapons->setWeaponsName(ui.pageOptions->WeaponsName->currentText(), true);
+	GoToPage(ID_PAGE_SELECTWEAPON);
+}
+
 void HWForm::GoToInfo()
 {
 	GoToPage(ID_PAGE_INFO);
@@ -138,44 +221,63 @@ void HWForm::GoToMultiplayer()
 	GoToPage(ID_PAGE_MULTIPLAYER);
 }
 
+void HWForm::GoToSaves()
+{
+	ui.pagePlayDemo->FillFromDir(PagePlayDemo::RT_Save);
+
+	GoToPage(ID_PAGE_DEMOS);
+}
+
 void HWForm::GoToDemos()
 {
-	QDir tmpdir;
-	tmpdir.cd(cfgdir->absolutePath());
-	tmpdir.cd("Demos");
-	tmpdir.setFilter(QDir::Files);
-	ui.pagePlayDemo->DemosList->clear();
-	ui.pagePlayDemo->DemosList->addItems(tmpdir.entryList(QStringList("*.hwd_" + cProtoVer))
-			.replaceInStrings(QRegExp("^(.*).hwd_" + cProtoVer), "\\1"));
+	ui.pagePlayDemo->FillFromDir(PagePlayDemo::RT_Demo);
+
 	GoToPage(ID_PAGE_DEMOS);
 }
 
 void HWForm::GoToNet()
 {
+	ui.pageNet->updateServersList();
+
 	GoToPage(ID_PAGE_NET);
 }
 
-void HWForm::OnPageShown(quint8 id)
+void HWForm::GoToNetServer()
+{
+	GoToPage(ID_PAGE_NETSERVER);
+}
+
+void HWForm::OnPageShown(quint8 id, quint8 lastid)
 {
 	if (id == ID_PAGE_MULTIPLAYER || id == ID_PAGE_NETCFG) {
 		QStringList tmNames=config->GetTeamsList();
 		TeamSelWidget* curTeamSelWidget;
-		id == ID_PAGE_MULTIPLAYER ? curTeamSelWidget=ui.pageMultiplayer->teamsSelect :
+		if(id == ID_PAGE_MULTIPLAYER) {
+		  curTeamSelWidget=ui.pageMultiplayer->teamsSelect;
+		} else {
 		  curTeamSelWidget=ui.pageNetGame->pNetTeamsWidget;
+		}
 		QList<HWTeam> teamsList;
 		for(QStringList::iterator it=tmNames.begin(); it!=tmNames.end(); it++) {
 		  HWTeam team(*it);
 		  team.LoadFromFile();
 		  teamsList.push_back(team);
 		}
-		curTeamSelWidget->resetPlayingTeams(teamsList);
+		if(lastid==ID_PAGE_SETUP) { // _TEAM
+		  if (editedTeam) {
+		    curTeamSelWidget->addTeam(*editedTeam);
+		  }
+		} else {
+		  curTeamSelWidget->resetPlayingTeams(teamsList);
+		}
 	}
 }
 
 void HWForm::GoToPage(quint8 id)
 {
+	quint8 lastid=ui.Pages->currentIndex();
 	PagesStack.push(ui.Pages->currentIndex());
-	OnPageShown(id);
+	OnPageShown(id, lastid);
 	ui.Pages->setCurrentIndex(id);
 }
 
@@ -185,10 +287,48 @@ void HWForm::GoBack()
 	  if(hwnet || pnetserver) NetDisconnect();
 	}
 	quint8 id = PagesStack.isEmpty() ? ID_PAGE_MAIN : PagesStack.pop();
-	OnPageShown(id);
+	OnPageShown(id, ui.Pages->currentIndex());
 	ui.Pages->setCurrentIndex(id);
 }
 
+void HWForm::btnExitPressed()
+{
+	eggTimer.start();
+}
+
+void HWForm::btnExitClicked()
+{
+	if (eggTimer.elapsed() < 3000)
+		close();
+	else
+	{
+		QPushButton * btn = findChild<QPushButton *>("imageButt");
+		if (btn)
+		{
+			btn->setIcon(QIcon(":/res/bonus.png"));
+		}
+	}
+}
+
+void HWForm::IntermediateSetup()
+{
+  quint8 id=ui.Pages->currentIndex();
+  TeamSelWidget* curTeamSelWidget;
+  if(id == ID_PAGE_MULTIPLAYER) {
+    curTeamSelWidget=ui.pageMultiplayer->teamsSelect;
+  } else {
+    curTeamSelWidget=ui.pageNetGame->pNetTeamsWidget;
+  }
+  QList<HWTeam> teams=curTeamSelWidget->getDontPlayingTeams();
+  QStringList tmnames;
+  for(QList<HWTeam>::iterator it = teams.begin(); it != teams.end(); ++it) {
+    tmnames+=it->TeamName;
+  }
+  UpdateTeamsLists(&tmnames); // FIXME: still need more work if teamname is updated while configuring
+
+  GoToPage(ID_PAGE_SETUP);
+}
+
 void HWForm::NewTeam()
 {
 	editedTeam = new HWTeam("unnamed");
@@ -209,6 +349,7 @@ void HWForm::TeamSave()
 	editedTeam->GetFromPage(this);
 	editedTeam->SaveToFile();
 	delete editedTeam;
+	editedTeam=0;
 	UpdateTeamsLists();
 	GoBack();
 }
@@ -216,12 +357,13 @@ void HWForm::TeamSave()
 void HWForm::TeamDiscard()
 {
 	delete editedTeam;
+	editedTeam=0;
 	GoBack();
 }
 
 void HWForm::SimpleGame()
 {
-	CreateGame(ui.pageLocalGame->gameCFG, 0);
+	CreateGame(ui.pageSimpleGame->gameCFG, 0, cDefaultAmmoStore->mid(10));
 	game->StartQuick();
 }
 
@@ -232,32 +374,30 @@ void HWForm::PlayDemo()
 	{
 		QMessageBox::critical(this,
 				tr("Error"),
-				tr("Please, select demo from the list above"),
+				tr("Please, select record from the list above"),
 				tr("OK"));
 		return ;
 	}
-	CreateGame(0, 0);
-	game->PlayDemo(cfgdir->absolutePath() + "/Demos/" + curritem->text() + ".hwd_" + cProtoVer);
+	CreateGame(0, 0, 0);
+	game->PlayDemo(curritem->data(Qt::UserRole).toString());
 }
 
-void HWForm::NetConnectServer()
+void HWForm::NetConnectServer(const QString & host, quint16 port)
 {
-  QListWidgetItem * curritem = ui.pageNet->pUdpClient->serversList->currentItem();
-  if (!curritem) {
-    QMessageBox::critical(this,
-			  tr("Error"),
-			  tr("Please, select server from the list above"),
-			  tr("OK"));
-    return ;
-  }
-  _NetConnect(curritem->text(), 46631, ui.pageNet->editNetNick->text());
+  _NetConnect(host, port, ui.pageOptions->editNetNick->text());
 }
 
 void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString & nick)
 {
+	if(hwnet) {
+		hwnet->Disconnect();
+		delete hwnet;
+		hwnet=0;
+	}
 	ui.pageNetGame->pChatWidget->clear();
 	hwnet = new HWNewNet(config, ui.pageNetGame->pGameCFG, ui.pageNetGame->pNetTeamsWidget);
-	connect(hwnet, SIGNAL(GameStateChanged(GameState)), this, SLOT(NetGameStateChanged(GameState)));
+
+	connect(hwnet, SIGNAL(AskForRunGame()), this, SLOT(CreateNetGame()));
 	connect(hwnet, SIGNAL(EnteredGame()), this, SLOT(NetGameEnter()));
 	connect(hwnet, SIGNAL(AddNetTeam(const HWTeam&)), this, SLOT(AddNetTeam(const HWTeam&)));
 
@@ -296,22 +436,47 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 		ui.pageNetGame->pNetTeamsWidget, SLOT(changeHHNum(const HWTeam&)));
 	connect(hwnet, SIGNAL(teamColorChanged(const HWTeam&)),
 		ui.pageNetGame->pNetTeamsWidget, SLOT(changeTeamColor(const HWTeam&)));
+	connect(hwnet, SIGNAL(ammoChanged(const QString&, const QString&)), ui.pageNetGame->pGameCFG, SLOT(setNetAmmo(const QString&, const QString&)));
 
 	hwnet->Connect(hostName, port, nick);
-	config->SaveOptions();
 }
 
 void HWForm::NetConnect()
 {
-  _NetConnect(ui.pageNet->editIP->text(), 46631, ui.pageNet->editNetNick->text());
+	HWHostPortDialog * hpd = new HWHostPortDialog(this);
+	hpd->leHost->setText(*netHost);
+	hpd->sbPort->setValue(netPort);
+
+	if (hpd->exec() == QDialog::Accepted)
+	{
+		config->SaveOptions();
+		delete netHost;
+		netHost = new QString(hpd->leHost->text());
+		netPort = hpd->sbPort->value();
+		_NetConnect(*netHost, netPort, ui.pageOptions->editNetNick->text());
+	}
 }
 
 void HWForm::NetStartServer()
 {
-  pnetserver = new HWNetServer;
-  pnetserver->StartServer();
-  _NetConnect("localhost", pnetserver->getRunningPort(), ui.pageNet->editNetNick->text());
-  pUdpServer = new HWNetUdpServer();
+	config->SaveOptions();
+
+	pnetserver = new HWNetServer;
+	if(!pnetserver->StartServer(ui.pageNetServer->sbPort->value()))
+	{
+		QMessageBox::critical(0, tr("Error"),
+					tr("Unable to start the server"));
+		delete pnetserver;
+		pnetserver = 0;
+		return;
+	}
+
+	_NetConnect("localhost", pnetserver->getRunningPort(), ui.pageOptions->editNetNick->text());
+
+	if (ui.pageNet->rbLocalGame->isChecked())
+		pRegisterServer = new HWNetUdpServer(0, ui.pageNetServer->leServerDescr->text(), ui.pageNetServer->sbPort->value());
+	else
+		pRegisterServer = new HWNetWwwServer(0, ui.pageNetServer->leServerDescr->text(), ui.pageNetServer->sbPort->value());
 }
 
 void HWForm::NetDisconnect()
@@ -322,7 +487,12 @@ void HWForm::NetDisconnect()
     hwnet=0;
   }
   if(pnetserver) {
-    pUdpServer->deleteLater();
+    if (pRegisterServer)
+    {
+      pRegisterServer->unregister();
+      pRegisterServer = 0;
+    }
+
     pnetserver->StopServer();
     delete pnetserver;
     pnetserver=0;
@@ -360,31 +530,41 @@ void HWForm::AddNetTeam(const HWTeam& team)
 
 void HWForm::StartMPGame()
 {
-	CreateGame(ui.pageMultiplayer->gameCFG, ui.pageMultiplayer->teamsSelect);
+	QString ammo=ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageMultiplayer->gameCFG->WeaponsName->currentText());
+
+	CreateGame(ui.pageMultiplayer->gameCFG, ui.pageMultiplayer->teamsSelect, ammo);
 
 	game->StartLocal();
 }
 
-void HWForm::NetGameStateChanged(GameState __attribute__((unused)) gameState)
-{
-  ui.pageNetGame->BtnGo->setText(QPushButton::tr("Go!"));
-  ui.pageNetGame->BtnGo->setEnabled(true);
-}
-
 void HWForm::GameStateChanged(GameState gameState)
 {
 	switch(gameState) {
 		case gsStarted: {
 			ui.pageGameStats->labelGameStats->setText("");
+			if (pRegisterServer)
+			{
+				pRegisterServer->unregister();
+				pRegisterServer = 0;
+			}
 			break;
 		}
 		case gsFinished: {
+			GoBack();
 			GoToPage(ID_PAGE_GAMESTATS);
 			break;
 		}
-		default: ;
+		default: {
+			quint8 id = ui.Pages->currentIndex();
+			if (id == ID_PAGE_INGAME) GoBack();
+		};
 	}
 
+	if (hwnet)
+	{
+		ui.pageNetGame->BtnGo->setText(QPushButton::tr("Go!"));
+		ui.pageNetGame->BtnGo->setEnabled(true);
+	}
 }
 
 void HWForm::AddStatText(const QString & msg)
@@ -415,12 +595,15 @@ void HWForm::GameStats(char type, const QString & info)
 	}
 }
 
-void HWForm::CreateGame(GameCFGWidget * gamecfg, TeamSelWidget* pTeamSelWidget)
+void HWForm::CreateGame(GameCFGWidget * gamecfg, TeamSelWidget* pTeamSelWidget, QString ammo)
 {
-	game = new HWGame(config, gamecfg, pTeamSelWidget);
+	game = new HWGame(config, gamecfg, ammo, pTeamSelWidget);
 	connect(game, SIGNAL(GameStateChanged(GameState)), this, SLOT(GameStateChanged(GameState)));
 	connect(game, SIGNAL(GameStats(char, const QString &)), this, SLOT(GameStats(char, const QString &)));
 	connect(game, SIGNAL(ErrorMessage(const QString &)), this, SLOT(ShowErrorMessage(const QString &)), Qt::QueuedConnection);
+	connect(game, SIGNAL(HaveRecord(bool, const QByteArray &)), this, SLOT(GetRecord(bool, const QByteArray &)));
+
+	GoToPage(ID_PAGE_INGAME);
 }
 
 void HWForm::ShowErrorMessage(const QString & msg)
@@ -429,3 +612,59 @@ void HWForm::ShowErrorMessage(const QString & msg)
 			"Hedgewars",
 			msg);
 }
+
+void HWForm::GetRecord(bool isDemo, const QByteArray & record)
+{
+	QString filename;
+	QByteArray demo = record;
+	if (isDemo)
+	{
+		demo.replace(QByteArray("\x02TL"), QByteArray("\x02TD"));
+		demo.replace(QByteArray("\x02TN"), QByteArray("\x02TD"));
+		filename = cfgdir->absolutePath() + "/Demos/LastRound.hwd_" + *cProtoVer;
+	} else
+	{
+		demo.replace(QByteArray("\x02TL"), QByteArray("\x02TS"));
+		demo.replace(QByteArray("\x02TN"), QByteArray("\x02TS"));
+		filename = cfgdir->absolutePath() + "/Saves/LastRound.hws_" + *cProtoVer;
+	}
+
+
+	QFile demofile(filename);
+	if (!demofile.open(QIODevice::WriteOnly))
+	{
+		ShowErrorMessage(tr("Cannot save record to file %1").arg(filename));
+		return ;
+	}
+	demofile.write(demo.constData(), demo.size());
+	demofile.close();
+}
+
+void HWForm::StartTraining()
+{
+	CreateGame(0, 0, 0);
+
+	game->StartTraining();
+}
+
+void HWForm::CreateNetGame()
+{
+	QString ammo;
+	if (pnetserver) {
+	  ammo=ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageNetGame->pGameCFG->WeaponsName->currentText());
+	} else {
+	  ammo=ui.pageNetGame->pGameCFG->getNetAmmo();
+	}
+	CreateGame(ui.pageNetGame->pGameCFG, ui.pageNetGame->pNetTeamsWidget, ammo);
+
+	connect(game, SIGNAL(SendNet(const QByteArray &)), hwnet, SLOT(SendNet(const QByteArray &)));
+	connect(hwnet, SIGNAL(FromNet(const QByteArray &)), game, SLOT(FromNet(const QByteArray &)));
+
+	game->StartNet();
+}
+
+void HWForm::closeEvent(QCloseEvent *event)
+{
+	config->SaveOptions();
+	event->accept();
+}
