@@ -20,15 +20,19 @@
 #define _NET_UDPSERVER_INCLUDED
 
 #include <QObject>
+#include "netregister.h"
 
 class QUdpSocket;
 
-class HWNetUdpServer : public QObject
+class HWNetUdpServer : public HWNetRegisterServer
 {
   Q_OBJECT
 
  public:
-  HWNetUdpServer(QObject *parent = 0);
+  HWNetUdpServer(QObject *parent, const QString & descr, quint16 port);
+
+ public slots:
+  void unregister();
 
  private slots:
   void onClientRead();
