@@ -1,6 +1,7 @@
 /*
  * Hedgewars, a worms-like game
  * Copyright (c) 2007 Igor Ulyanov <iulyanov@gmail.com>
+ * Copyright (c) 2007 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,30 +20,27 @@
 #ifndef _NET_UDPWIDGET_INCLUDED
 #define _NET_UDPWIDGET_INCLUDED
 
-#include <QWidget>
-#include <QVBoxLayout>
+#include "netserverslist.h"
 
 class QUdpSocket;
-class QListWidget;
 
-class HWNetUdpWidget : public QWidget
+class HWNetUdpModel : public HWNetServersModel
 {
   Q_OBJECT
 
- public:
-  HWNetUdpWidget(QWidget *parent = 0);
+public:
+	HWNetUdpModel(QObject *parent = 0);
 
-  QListWidget* serversList;
+	QVariant data(const QModelIndex &index, int role) const;
 
- public slots:
-  void updateList();
+public slots:
+	void updateList();
 
- private slots:
-  void onClientRead();
+private slots:
+	void onClientRead();
 
- private:
-  QVBoxLayout mainLayout;
-  QUdpSocket* pUdpSocket;
+private:
+	QUdpSocket* pUdpSocket;
 };
 
 #endif // _NET_UDPWIDGET_INCLUDED
