@@ -28,6 +28,10 @@
 #include <QApplication>
 #include <QSpinBox>
 #include <QTextEdit>
+#include <QRadioButton>
+#include <QTableView>
+#include <QMessageBox>
+#include <QHeaderView>
 
 #include "pages.h"
 #include "sdlkeys.h"
@@ -39,79 +43,61 @@
 #include "mapContainer.h"
 #include "about.h"
 #include "fpsedit.h"
+#include "netserverslist.h"
 #include "netudpwidget.h"
+#include "netwwwwidget.h"
 #include "chatwidget.h"
+#include "SDLs.h"
+#include "playrecordpage.h"
+#include "selectWeapon.h"
 
-PageMain::PageMain(QWidget* parent) : QWidget(parent)
+PageMain::PageMain(QWidget* parent) : 
+  AbstractPage(parent)
 {
-	QFont * font14 = new QFont("MS Shell Dlg", 14);
 	QGridLayout * pageLayout = new QGridLayout(this);
 	pageLayout->setMargin(25);
 	pageLayout->setColumnStretch(0, 1);
 	pageLayout->setColumnStretch(1, 2);
 	pageLayout->setColumnStretch(2, 1);
 
-	BtnSinglePlayer = new QPushButton(this);
+	BtnSinglePlayer = addButton(tr("Single Player"), pageLayout, 0, 1);
 	BtnSinglePlayer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	BtnSinglePlayer->setFont(*font14);
-	BtnSinglePlayer->setText(QPushButton::tr("Single Player"));
-	pageLayout->addWidget(BtnSinglePlayer, 1, 1);
 
-	BtnMultiplayer = new QPushButton(this);
+	BtnMultiplayer = addButton(tr("Multiplayer"), pageLayout, 1, 1);
 	BtnMultiplayer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	BtnMultiplayer->setFont(*font14);
-	BtnMultiplayer->setText(QPushButton::tr("Multiplayer"));
-	pageLayout->addWidget(BtnMultiplayer, 2, 1);
 
-	BtnNet = new QPushButton(this);
+	BtnNet = addButton(tr("Net game"), pageLayout, 2, 1);
 	BtnNet->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	BtnNet->setFont(*font14);
-	BtnNet->setText(QPushButton::tr("Net game"));
-	pageLayout->addWidget(BtnNet, 3, 1);
 
-	BtnDemos = new QPushButton(this);
+	BtnLoad = addButton(tr("Saved games"), pageLayout, 3, 1);
+	BtnLoad->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+
+	BtnDemos = addButton(tr("Demos"), pageLayout, 4, 1);
 	BtnDemos->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	BtnDemos->setFont(*font14);
-	BtnDemos->setText(QPushButton::tr("Demos"));
-	pageLayout->addWidget(BtnDemos, 4, 1);
 
-	BtnSetup = new QPushButton(this);
+	BtnSetup = addButton(tr("Setup"), pageLayout, 5, 1);
 	BtnSetup->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	BtnSetup->setFont(*font14);
-	BtnSetup->setText(QPushButton::tr("Setup"));
-	pageLayout->addWidget(BtnSetup, 5, 1);
 
-	BtnInfo = new QPushButton(this);
+	BtnInfo = addButton(tr("About"), pageLayout, 6, 1);
 	BtnInfo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	BtnInfo->setFont(*font14);
-	BtnInfo->setText(QPushButton::tr("About"));
-	pageLayout->addWidget(BtnInfo, 6, 1);
 
-	BtnExit = new QPushButton(parent);
+	BtnExit = addButton(tr("Exit"), pageLayout, 7, 1);
 	BtnExit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	BtnExit->setFont(*font14);
-	BtnExit->setText(QPushButton::tr("Exit"));
-	pageLayout->addWidget(BtnExit, 7, 1);
 }
 
-PageLocalGame::PageLocalGame(QWidget* parent) : QWidget(parent)
+PageSimpleGame::PageSimpleGame(QWidget* parent) : 
+  AbstractPage(parent)
 {
-	QFont * font14 = new QFont("MS Shell Dlg", 14);
 	QVBoxLayout * pageLayout = new QVBoxLayout(this);
 	QHBoxLayout * topLayout = new QHBoxLayout();
 	QHBoxLayout * bottomLayout = new QHBoxLayout();
 	pageLayout->addLayout(topLayout, 100);
 	pageLayout->addLayout(bottomLayout, 0);
 
-	BtnBack = new QPushButton(this);
-	BtnBack->setFont(*font14);
-	BtnBack->setText(QPushButton::tr("Back"));
-	bottomLayout->addWidget(BtnBack, 100);
+	BtnBack = addButton(tr("Back"), bottomLayout, 100);
 	bottomLayout->addStretch(100);
-	BtnSimpleGame = new	QPushButton(this);
-	BtnSimpleGame->setFont(*font14);
-	BtnSimpleGame->setText(QPushButton::tr("Simple Game"));
-	bottomLayout->addWidget(BtnSimpleGame, 100);
+
+	BtnSimpleGame = addButton(tr("Simple Game"), bottomLayout, 100);
 
 	gameCFG = new GameCFGWidget(this);
 	topLayout->addStretch(100);
@@ -119,9 +105,9 @@ PageLocalGame::PageLocalGame(QWidget* parent) : QWidget(parent)
 	topLayout->addStretch(100);
 }
 
-PageEditTeam::PageEditTeam(QWidget* parent) : QWidget(parent)
+PageEditTeam::PageEditTeam(QWidget* parent) : 
+  AbstractPage(parent)
 {
-	QFont * font14 = new QFont("MS Shell Dlg", 14);
 	QGridLayout * pageLayout = new QGridLayout(this);
 	pageLayout->setColumnStretch(0, 1);
 	pageLayout->setColumnMinimumWidth(0, 150);
@@ -142,7 +128,7 @@ PageEditTeam::PageEditTeam(QWidget* parent) : QWidget(parent)
 
 	pageLayout->addWidget(GBoxTeam, 0, 0);
 
-	GBoxHedgehogs = new	QGroupBox(this);
+	GBoxHedgehogs = new QGroupBox(this);
 	GBoxHedgehogs->setTitle(QGroupBox::tr("Team Members"));
 	GBoxHedgehogs->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 	QGridLayout * GBHLayout = new QGridLayout(GBoxHedgehogs);
@@ -155,12 +141,9 @@ PageEditTeam::PageEditTeam(QWidget* parent) : QWidget(parent)
 	}
 	pageLayout->addWidget(GBoxHedgehogs, 1, 0, 2, 1);
 
-	BtnTeamDiscard = new QPushButton(this);
-	BtnTeamDiscard->setFont(*font14);
-	BtnTeamDiscard->setText(QPushButton::tr("Discard"));
-	pageLayout->addWidget(BtnTeamDiscard, 4, 0);
+	BtnTeamDiscard = addButton(tr("Discard"), pageLayout, 4, 0);
 
-	GBoxBinds =	new QGroupBox(this);
+	GBoxBinds = new QGroupBox(this);
 	GBoxBinds->setTitle(QGroupBox::tr("Key binds"));
 	QGridLayout * GBBLayout = new QGridLayout(GBoxBinds);
 	BindsBox = new QToolBox(GBoxBinds);
@@ -200,7 +183,7 @@ PageEditTeam::PageEditTeam(QWidget* parent) : QWidget(parent)
 		widind++;
 	}
 
-	GBoxGrave =	new QGroupBox(this);
+	GBoxGrave = new QGroupBox(this);
 	GBoxGrave->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 	GBoxGrave->setTitle(QGroupBox::tr("Grave"));
 	QGridLayout * GBGLayout = new QGridLayout(GBoxGrave);
@@ -237,28 +220,25 @@ PageEditTeam::PageEditTeam(QWidget* parent) : QWidget(parent)
 	CBFort = new QComboBox(GBoxFort);
 	CBFort->setMaxCount(65535);
 	GBFLayout->addWidget(CBFort, 0, 0);
-	FortPreview	= new SquareLabel(GBoxFort);
+	FortPreview = new SquareLabel(GBoxFort);
 	FortPreview->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 	FortPreview->setPixmap(QPixmap());
 	GBFLayout->addWidget(FortPreview, 1, 0);
 	pageLayout->addWidget(GBoxFort, 2, 2, 1, 2);
 
-	BtnTeamSave	= new QPushButton(this);
-	BtnTeamSave->setFont(*font14);
-	BtnTeamSave->setText(QPushButton::tr("Save"));
-	pageLayout->addWidget(BtnTeamSave, 4, 2, 1, 2);
+	BtnTeamSave = addButton(tr("Save"), pageLayout, 4, 2, 1, 2);
 
 	QDir tmpdir;
 	tmpdir.cd(datadir->absolutePath());
 	tmpdir.cd("Forts");
 	tmpdir.setFilter(QDir::Files);
 
-	CBFort->addItems(tmpdir.entryList(QStringList("*L.png")).replaceInStrings(QRegExp("^(.*)L.png"), "\\1"));
+	CBFort->addItems(tmpdir.entryList(QStringList("*L.png")).replaceInStrings(QRegExp("^(.*)L\\.png"), "\\1"));
 	tmpdir.cd("../Graphics/Graves");
 	QStringList list = tmpdir.entryList(QStringList("*.png"));
 	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it )
 	{
-		CBGrave->addItem((*it).replace(QRegExp("^(.*).png"), "\\1"));
+		CBGrave->addItem((*it).replace(QRegExp("^(.*)\\.png"), "\\1"));
 	}
 
 	connect(CBGrave, SIGNAL(activated(const QString &)), this, SLOT(CBGrave_activated(const QString &)));
@@ -284,62 +264,34 @@ void PageEditTeam::CBTeamLvl_activated(int id)
 	LevelPict->setPixmap(pix);
 }
 
-PageMultiplayer::PageMultiplayer(QWidget* parent) : QWidget(parent)
+PageMultiplayer::PageMultiplayer(QWidget* parent) : 
+  AbstractPage(parent)
 {
-	QFont * font14 = new QFont("MS Shell Dlg", 14);
 	QGridLayout * pageLayout = new QGridLayout(this);
 
-	BtnBack = new QPushButton(this);
-	BtnBack->setFont(*font14);
-	BtnBack->setText(QPushButton::tr("Back"));
-	pageLayout->addWidget(BtnBack, 1, 0);
+	BtnBack = addButton(tr("Back"), pageLayout, 1, 0);
 
 	gameCFG = new GameCFGWidget(this);
 	pageLayout->addWidget(gameCFG, 0, 0, 1, 2);
+
 	teamsSelect = new TeamSelWidget(this);
 	pageLayout->addWidget(teamsSelect, 0, 2, 1, 2);
 
-	BtnStartMPGame = new QPushButton(this);
-	BtnStartMPGame->setFont(*font14);
-	BtnStartMPGame->setText(QPushButton::tr("Start"));
-	pageLayout->addWidget(BtnStartMPGame, 1, 3);
+	BtnStartMPGame = addButton(tr("Start"), pageLayout, 1, 3);
 }
 
-PagePlayDemo::PagePlayDemo(QWidget* parent) : QWidget(parent)
+PageOptions::PageOptions(QWidget* parent) : 
+  AbstractPage(parent)
 {
-	QFont * font14 = new QFont("MS Shell Dlg", 14);
-	QGridLayout * pageLayout = new QGridLayout(this);
-	pageLayout->setColumnStretch(0, 1);
-	pageLayout->setColumnStretch(1, 2);
-	pageLayout->setColumnStretch(2, 1);
-
-	BtnBack = new QPushButton(this);
-	BtnBack->setFont(*font14);
-	BtnBack->setText(QPushButton::tr("Back"));
-	pageLayout->addWidget(BtnBack, 1, 0);
-
-	BtnPlayDemo	= new QPushButton(this);
-	BtnPlayDemo->setGeometry(QRect(240,	330, 161, 41));
-	BtnPlayDemo->setFont(*font14);
-	BtnPlayDemo->setText(QPushButton::tr("Play demo"));
-	pageLayout->addWidget(BtnPlayDemo, 1, 2);
-
-	DemosList =	new QListWidget(this);
-	DemosList->setGeometry(QRect(170, 10, 311, 311));
-	pageLayout->addWidget(DemosList, 0, 1);
-}
-
-PageOptions::PageOptions(QWidget* parent) : QWidget(parent)
-{
-	QFont * font14 = new QFont("MS Shell Dlg", 14);
 	QGridLayout * pageLayout = new QGridLayout(this);
 	pageLayout->setColumnStretch(0, 100);
 	pageLayout->setColumnStretch(1, 100);
 	pageLayout->setColumnStretch(2, 100);
 	pageLayout->setRowStretch(0, 0);
 	pageLayout->setRowStretch(1, 0);
-	pageLayout->setRowStretch(2, 100);
-	pageLayout->setRowStretch(3, 0);
+	pageLayout->setRowStretch(2, 0);
+	pageLayout->setRowStretch(3, 100);
+	pageLayout->setRowStretch(4, 0);
 
 	groupBox = new QGroupBox(this);
 	groupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
@@ -347,23 +299,18 @@ PageOptions::PageOptions(QWidget* parent) : QWidget(parent)
 	pageLayout->addWidget(groupBox, 0, 0, 1, 3);
 
 	QGridLayout * GBTlayout = new QGridLayout(groupBox);
-	BtnNewTeam = new QPushButton(groupBox);
-	BtnNewTeam->setFont(*font14);
-	BtnNewTeam->setText(QPushButton::tr("New team"));
-	GBTlayout->addWidget(BtnNewTeam, 0, 0);
+
+	BtnNewTeam = addButton(tr("New team"), GBTlayout, 0, 0);
 
 	CBTeamName = new QComboBox(groupBox);
 	GBTlayout->addWidget(CBTeamName, 0, 1);
 
-	BtnEditTeam	= new QPushButton(groupBox);
-	BtnEditTeam->setFont(*font14);
-	BtnEditTeam->setText(QPushButton::tr("Edit team"));
-	GBTlayout->addWidget(BtnEditTeam, 0, 2);
+	BtnEditTeam = addButton(tr("Edit team"), GBTlayout, 0, 2);
 
 	AGGroupBox = new QGroupBox(this);
 	AGGroupBox->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 	AGGroupBox->setTitle(QGroupBox::tr("Audio/Graphic options"));
-	pageLayout->addWidget(AGGroupBox, 1, 1);
+	pageLayout->addWidget(AGGroupBox, 2, 1);
 
 	QVBoxLayout * GBAlayout = new QVBoxLayout(AGGroupBox);
 	QHBoxLayout * GBAreslayout = new QHBoxLayout(0);
@@ -372,10 +319,8 @@ PageOptions::PageOptions(QWidget* parent) : QWidget(parent)
 	GBAreslayout->addWidget(resolution);
 
 	CBResolution = new QComboBox(AGGroupBox);
-	CBResolution->addItem("640x480");
-	CBResolution->addItem("800x600");
-	CBResolution->addItem("1024x768");
-	CBResolution->addItem("1280x1024");
+	SDLInteraction sdli;
+	CBResolution->addItems(sdli.getResolutions());
 	GBAreslayout->addWidget(CBResolution);
 	GBAlayout->addLayout(GBAreslayout);
 
@@ -397,18 +342,42 @@ PageOptions::PageOptions(QWidget* parent) : QWidget(parent)
 	CBShowFPS->setText(QCheckBox::tr("Show FPS"));
 	GBAlayout->addWidget(CBShowFPS);
 
+	CBAltDamage = new QCheckBox(AGGroupBox);
+	CBAltDamage->setText(QCheckBox::tr("Alternative damage show"));
+	GBAlayout->addWidget(CBAltDamage);
+
 	fpsedit = new FPSEdit(AGGroupBox);
 	GBAfpslayout->addWidget(fpsedit);
 
-	BtnSaveOptions = new QPushButton(this);
-	BtnSaveOptions->setFont(*font14);
-	BtnSaveOptions->setText(QPushButton::tr("Save"));
-	pageLayout->addWidget(BtnSaveOptions, 4, 2);
+	BtnSaveOptions = addButton(tr("Save"), pageLayout, 4, 2);
 
-	BtnBack = new QPushButton(this);
-	BtnBack->setFont(*font14);
-	BtnBack->setText(QPushButton::tr("Back"));
-	pageLayout->addWidget(BtnBack, 4, 0);
+	BtnBack = addButton(tr("Back"), pageLayout, 4, 0);
+
+	QGroupBox* groupWeapons = new QGroupBox(this);
+	groupWeapons->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+	groupWeapons->setTitle(QGroupBox::tr("Weapons"));
+	pageLayout->addWidget(groupWeapons, 1, 0, 1, 3);
+	QGridLayout * WeaponsLayout = new QGridLayout(groupWeapons);
+
+	WeaponsButt = addButton(tr("Weapons set"), WeaponsLayout, 0, 0);
+	WeaponsName = new QComboBox(this);
+	WeaponsLayout->addWidget(WeaponsName, 0, 1);
+	WeaponEdit = addButton(tr("Edit"), WeaponsLayout, 0, 2);
+
+	NNGroupBox = new QGroupBox(this);
+	NNGroupBox->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+	NNGroupBox->setTitle(QGroupBox::tr("Net options"));
+	pageLayout->addWidget(NNGroupBox, 2, 2);
+
+	QGridLayout * GBNlayout = new QGridLayout(NNGroupBox);
+	labelNN = new QLabel(NNGroupBox);
+	labelNN->setText(QLabel::tr("Net nick"));
+	GBNlayout->addWidget(labelNN, 0, 0);
+
+	editNetNick = new QLineEdit(NNGroupBox);
+	editNetNick->setMaxLength(20);
+	editNetNick->setText(QLineEdit::tr("unnamed"));
+	GBNlayout->addWidget(editNetNick, 0, 1);
 }
 
 PageNet::PageNet(QWidget* parent) : QWidget(parent)
@@ -417,63 +386,156 @@ PageNet::PageNet(QWidget* parent) : QWidget(parent)
 	QGridLayout * pageLayout = new QGridLayout(this);
 	pageLayout->setColumnStretch(0, 1);
 	pageLayout->setColumnStretch(1, 1);
+	pageLayout->setColumnStretch(2, 1);
 
 	BtnNetSvrStart = new QPushButton(this);
 	BtnNetSvrStart->setFont(*font14);
 	BtnNetSvrStart->setText(QPushButton::tr("Start server"));
-	pageLayout->addWidget(BtnNetSvrStart, 0, 1);
+	pageLayout->addWidget(BtnNetSvrStart, 3, 2);
 
-	NNGroupBox = new QGroupBox(this);
-	NNGroupBox->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
-	NNGroupBox->setTitle(QGroupBox::tr("Net options"));
-	pageLayout->addWidget(NNGroupBox, 0, 0);
+	QGroupBox * NetTypeGroupBox = new QGroupBox(this);
+	NetTypeGroupBox->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+	NetTypeGroupBox->setTitle(QGroupBox::tr("Servers list"));
+	pageLayout->addWidget(NetTypeGroupBox, 0, 1);
 
-	QGridLayout * GBNlayout = new QGridLayout(NNGroupBox);
-	labelNN = new QLabel(NNGroupBox);
-	labelNN->setText(QLabel::tr("Net nick"));
-	GBNlayout->addWidget(labelNN, 0, 0);
+	QVBoxLayout * GBTlayout = new QVBoxLayout(NetTypeGroupBox);
+	rbLocalGame = new QRadioButton(NetTypeGroupBox);
+	rbLocalGame->setText(tr("Local"));
+	rbLocalGame->setChecked(true);
+	GBTlayout->addWidget(rbLocalGame);
+	rbInternetGame = new QRadioButton(NetTypeGroupBox);
+	rbInternetGame->setText(tr("Internet"));
+	GBTlayout->addWidget(rbInternetGame);
 
-	editNetNick	= new QLineEdit(NNGroupBox);
-	editNetNick->setMaxLength(20);
-	editNetNick->setText(QLineEdit::tr("unnamed"));
-	GBNlayout->addWidget(editNetNick, 0, 1);
-
-	QGroupBox * ConnGroupBox = new QGroupBox(this);
+	ConnGroupBox = new QGroupBox(this);
 	ConnGroupBox->setTitle(QGroupBox::tr("Net game"));
 	pageLayout->addWidget(ConnGroupBox, 2, 0, 1, 3);
-	QGridLayout * GBClayout = new QGridLayout(ConnGroupBox);
-	GBClayout->setColumnStretch(0, 0);
+	GBClayout = new QGridLayout(ConnGroupBox);
+	GBClayout->setColumnStretch(0, 1);
 	GBClayout->setColumnStretch(1, 1);
 	GBClayout->setColumnStretch(2, 1);
-	GBClayout->setRowStretch(0, 0);
-	GBClayout->setRowStretch(1, 0);
-	GBClayout->setRowStretch(2, 1);
 
-	labelIP = new QLabel(ConnGroupBox);
-	labelIP->setText(QLabel::tr("Server address"));
-	GBClayout->addWidget(labelIP, 0, 0);
-
-	editIP = new QLineEdit(ConnGroupBox);
-	editIP->setMaxLength(50);
-	GBClayout->addWidget(editIP, 0, 1);
-
-	BtnNetConnect = new	QPushButton(ConnGroupBox);
+	BtnNetConnect = new QPushButton(ConnGroupBox);
 	BtnNetConnect->setFont(*font14);
 	BtnNetConnect->setText(QPushButton::tr("Connect"));
-	GBClayout->addWidget(BtnNetConnect, 0, 2);
+	GBClayout->addWidget(BtnNetConnect, 2, 2);
 
-	pUdpClient=new HWNetUdpWidget(ConnGroupBox);
-	GBClayout->addWidget(pUdpClient, 1, 0, 2, 2);
+	tvServersList = new QTableView(ConnGroupBox);
+	tvServersList->setSelectionBehavior(QAbstractItemView::SelectRows);
+	GBClayout->addWidget(tvServersList, 1, 0, 1, 3);
 
-	pUpdateUdpButt = new QPushButton(ConnGroupBox);
-	pUpdateUdpButt->setFont(*font14);
-	pUpdateUdpButt->setText(QPushButton::tr("Update"));
-	GBClayout->addWidget(pUpdateUdpButt, 1, 2);
+	BtnUpdateSList = new QPushButton(ConnGroupBox);
+	BtnUpdateSList->setFont(*font14);
+	BtnUpdateSList->setText(QPushButton::tr("Update"));
+	GBClayout->addWidget(BtnUpdateSList, 2, 0);
+
+	BtnSpecifyServer = new QPushButton(ConnGroupBox);
+	BtnSpecifyServer->setFont(*font14);
+	BtnSpecifyServer->setText(QPushButton::tr("Specify"));
+	GBClayout->addWidget(BtnSpecifyServer, 2, 1);
 
 	BtnBack = new QPushButton(this);
 	BtnBack->setFont(*font14);
 	BtnBack->setText(QPushButton::tr("Back"));
 	pageLayout->addWidget(BtnBack, 3, 0);
+
+	connect(rbLocalGame, SIGNAL(toggled(bool)), this, SLOT(updateServersList()));
+	connect(BtnNetConnect, SIGNAL(clicked()), this, SLOT(slotConnect()));
+}
+
+void PageNet::updateServersList()
+{
+	if (rbLocalGame->isChecked())
+		tvServersList->setModel(new HWNetUdpModel(tvServersList));
+	else
+		tvServersList->setModel(new HWNetWwwModel(tvServersList));
+
+	tvServersList->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
+
+	static_cast<HWNetServersModel *>(tvServersList->model())->updateList();
+
+	connect(BtnUpdateSList, SIGNAL(clicked()), static_cast<HWNetServersModel *>(tvServersList->model()), SLOT(updateList()));
+	connect(tvServersList, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(slotConnect()));
+}
+
+void PageNet::slotConnect()
+{
+	HWNetServersModel * model = static_cast<HWNetServersModel *>(tvServersList->model());
+	QModelIndex mi = tvServersList->currentIndex();
+	if(!mi.isValid())
+	{
+		QMessageBox::information(this, tr("Error"), tr("Please, select server from the list above"));
+		return;
+	}
+	QString host = model->index(mi.row(), 1).data().toString();
+	quint16 port = model->index(mi.row(), 2).data().toUInt();
+
+	emit connectClicked(host, port);
+}
+
+PageNetServer::PageNetServer(QWidget* parent) : QWidget(parent)
+{
+	QFont * font14 = new QFont("MS Shell Dlg", 14);
+	QGridLayout * pageLayout = new QGridLayout(this);
+	pageLayout->setColumnStretch(0, 1);
+	pageLayout->setColumnStretch(1, 1);
+	pageLayout->setColumnStretch(2, 1);
+
+	pageLayout->setRowStretch(0, 1);
+	pageLayout->setRowStretch(1, 0);
+
+	BtnBack = new QPushButton(this);
+	BtnBack->setFont(*font14);
+	BtnBack->setText(QPushButton::tr("Back"));
+	pageLayout->addWidget(BtnBack, 1, 0);
+
+	BtnStart = new QPushButton(this);
+	BtnStart->setFont(*font14);
+	BtnStart->setText(QPushButton::tr("Start"));
+	pageLayout->addWidget(BtnStart, 1, 2);
+
+	QWidget * wg = new QWidget(this);
+	pageLayout->addWidget(wg, 0, 0, 1, 3);
+
+	QGridLayout * wgLayout = new QGridLayout(wg);
+	wgLayout->setColumnStretch(0, 1);
+	wgLayout->setColumnStretch(1, 3);
+	wgLayout->setColumnStretch(2, 1);
+
+	wgLayout->setRowStretch(0, 0);
+	wgLayout->setRowStretch(1, 1);
+
+	QGroupBox * gb = new QGroupBox(wg);
+	wgLayout->addWidget(gb, 0, 1);
+
+	QGridLayout * gbLayout = new QGridLayout(gb);
+
+	labelSD = new QLabel(gb);
+	labelSD->setText(QLabel::tr("Server name:"));
+	gbLayout->addWidget(labelSD, 0, 0);
+
+	leServerDescr = new QLineEdit(gb);
+	gbLayout->addWidget(leServerDescr, 0, 1);
+
+	labelPort = new QLabel(gb);
+	labelPort->setText(QLabel::tr("Server port:"));
+	gbLayout->addWidget(labelPort, 1, 0);
+
+	sbPort = new QSpinBox(gb);
+	sbPort->setMinimum(0);
+	sbPort->setMaximum(65535);
+	gbLayout->addWidget(sbPort, 1, 1);
+
+	BtnDefault = new QPushButton(gb);
+	BtnDefault->setText(QPushButton::tr("default"));
+	gbLayout->addWidget(BtnDefault, 1, 2);
+
+	connect(BtnDefault, SIGNAL(clicked()), this, SLOT(setDefaultPort()));
+}
+
+void PageNetServer::setDefaultPort()
+{
+	sbPort->setValue(46631);
 }
 
 PageNetGame::PageNetGame(QWidget* parent) : QWidget(parent)
@@ -503,7 +565,7 @@ PageNetGame::PageNetGame(QWidget* parent) : QWidget(parent)
 	BtnBack->setText(QPushButton::tr("Back"));
 	pageLayout->addWidget(BtnBack, 2, 0);
 
-	BtnGo	= new QPushButton(this);
+	BtnGo = new QPushButton(this);
 	BtnGo->setFont(*font14);
 	BtnGo->setText(QPushButton::tr("Go!"));
 	BtnGo->setEnabled(false);
@@ -544,3 +606,73 @@ PageGameStats::PageGameStats(QWidget* parent) : QWidget(parent)
 	labelGameStats->setTextFormat(Qt::RichText);
 	pageLayout->addWidget(labelGameStats, 0, 0, 1, 3);
 }
+
+PageSinglePlayer::PageSinglePlayer(QWidget* parent) : QWidget(parent)
+{
+	QFont * font14 = new QFont("MS Shell Dlg", 14);
+	QGridLayout * pageLayout = new QGridLayout(this);
+	pageLayout->setMargin(25);
+	pageLayout->setColumnStretch(0, 1);
+	pageLayout->setColumnStretch(1, 2);
+	pageLayout->setColumnStretch(2, 1);
+	pageLayout->setRowStretch(0, 1);
+	pageLayout->setRowStretch(3, 1);
+
+	BtnSimpleGamePage = new QPushButton(this);
+	BtnSimpleGamePage->setFont(*font14);
+	BtnSimpleGamePage->setText(QPushButton::tr("Simple Game"));
+	pageLayout->addWidget(BtnSimpleGamePage, 1, 1);
+
+	BtnTrainPage = new QPushButton(this);
+	BtnTrainPage->setFont(*font14);
+	BtnTrainPage->setText(QPushButton::tr("Training"));
+	pageLayout->addWidget(BtnTrainPage, 2, 1);
+
+	BtnBack = new QPushButton(this);
+	BtnBack->setFont(*font14);
+	BtnBack->setText(QPushButton::tr("Back"));
+	pageLayout->addWidget(BtnBack, 4, 0);
+
+}
+
+PageTraining::PageTraining(QWidget* parent) : QWidget(parent)
+{
+	QFont * font14 = new QFont("MS Shell Dlg", 14);
+	QGridLayout * pageLayout = new QGridLayout(this);
+	pageLayout->setMargin(25);
+	pageLayout->setColumnStretch(0, 1);
+	pageLayout->setColumnStretch(1, 2);
+	pageLayout->setColumnStretch(2, 1);
+
+	BtnStartTrain = new QPushButton(this);
+	BtnStartTrain->setFont(*font14);
+	BtnStartTrain->setText(QPushButton::tr("Go!"));
+	pageLayout->addWidget(BtnStartTrain, 1, 2);
+
+	BtnBack = new QPushButton(this);
+	BtnBack->setFont(*font14);
+	BtnBack->setText(QPushButton::tr("Back"));
+	pageLayout->addWidget(BtnBack, 1, 0);
+}
+
+PageSelectWeapon::PageSelectWeapon(QWidget* parent) : 
+  AbstractPage(parent)
+{
+	QGridLayout * pageLayout = new QGridLayout(this);
+	pageLayout->setMargin(25);
+        
+        pWeapons=new SelWeaponWidget(20, this);
+	pageLayout->addWidget(pWeapons, 0, 0, 1, 4);
+
+	BtnBack = addButton(tr("Back"), pageLayout, 1, 0);
+	BtnDefault = addButton(tr("Default"), pageLayout, 1, 1);
+	BtnDelete = addButton(tr("Delete"), pageLayout, 1, 2);
+	BtnSave = addButton(tr("Save"), pageLayout, 1, 3);
+}
+
+PageInGame::PageInGame(QWidget* parent) : 
+  AbstractPage(parent)
+{
+	QLabel * label = new QLabel(this);
+	label->setText("In game...");
+}
